local utils = require 'utils'
local ngx = require "ngx"

local function read_file(path)
    -- r read mode and b binary mode
    local file = io.open(path, "rb")
    if not file then
        return nil
    end
    -- *a or *all reads the whole file
    local content = file:read("*a")
    file:close()
    return content
end

local path = ngx.req.get_uri_args().path
if path == nil then
    ngx.exit(ngx.HTTP_BAD_REQUEST)
end

if not utils.isPathReadable(path) then
    ngx.exit(ngx.HTTP_FORBIDDEN)
end

local content = read_file(path)
if content then
    local segments = utils.split(path, '/')
    local filename = segments[#segments]
    ngx.header["Content-Type"] = "application/octet-stream"
    if filename then
        ngx.header["Content-Disposition"] = 'attachment; filename="' .. filename .. '"'
    end
    ngx.print(content)
    ngx.exit(ngx.HTTP_OK)
else
    ngx.exit(ngx.HTTP_BAD_REQUEST)
end
