local utils = require 'utils'
local cjson = require 'cjson.safe'
local errType = require 'errType'
local config = require 'config'

local ArcView_ArchivesConfigCreate = {}

function ArcView_ArchivesConfigCreate.invoke(args)
    local root_dir, route_api

    if not (config and config.arcview and config.arcview.location) then
        return nil, errType.codes.InvalidParams,
        {message = 'Invalid arcview configuration.'}
    end

    if args ~= nil then
        if utils.isArray(args) then
            root_dir = args[1]
            route_api = args[2]
        else
            root_dir = args.archives_root_dir
            route_api = args.route_api
        end
    end

    if root_dir == nil or type(root_dir) ~= 'string' then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'root_dir\' is mandatory and must be a string.'}
    end
    if route_api ~= nil and type(route_api) ~= 'string' then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'route_api\' is mandatory and must be a string.'}
    end

    local rexarcParams = {}

    if root_dir ~= nil then
        if not utils.fileExists(root_dir) then
            return nil, errType.codes.ServerError, {message = 'Target directory ' .. root_dir .. ' does not exist.'}
        end

        table.insert(rexarcParams, '--archives_root_dir')
        table.insert(rexarcParams, root_dir)
    end
    if route_api ~= nil then
        table.insert(rexarcParams, '--route_api')
        table.insert(rexarcParams, route_api)
    end

    local app_dir = config.arcview.location
    local out, err, errData = utils.shell_posix('sudo', nil, '/usr/bin/python3', app_dir .. 'manage_bootstrap.py', 'rex_archives_configuration_create', unpack(rexarcParams))
    if err ~= nil then
        return nil, err, errData
    end

    return out
end

return ArcView_ArchivesConfigCreate
