local utils = require 'utils'
local cjson = require 'cjson.safe'
local errType = require 'errType'
local config = require 'config'

local ArcView_CheckInitialized = {}

function ArcView_CheckInitialized.invoke(args)
    local config_type

    if not (config and config.arcview and config.arcview.location) then
        return nil, errType.codes.InvalidParams,
        {message = 'Invalid arcview configuration.'}
    end

    if args ~= nil then
        if utils.isArray(args) then
            config_type = args[1]
        else
            config_type = args.type
        end
    end

    if config_type == nil or type(config_type) ~= 'string' then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'type\' is mandatory and must be a string.'}
    end

    local rexarcParams = {}

    if config_type ~= nil then
        table.insert(rexarcParams, '--type')
        table.insert(rexarcParams, config_type)
    end

    local app_dir = config.arcview.location
    local out, err, errData = utils.shell_posix('python3', nil, app_dir .. 'manage_bootstrap.py', 'configuration_check', unpack(rexarcParams))
    if err ~= nil then
        return nil, err, errData
    end

    return out
end

return ArcView_CheckInitialized
