local utils = require 'utils'
local cjson = require 'cjson.safe'
local errType = require 'errType'
local config = require 'config'

local ArcView_DbConfigCreate = {}

function ArcView_DbConfigCreate.invoke(args)
    local database, host, port, user,
    password, name, schema, route_api

    if not (config and config.arcview and config.arcview.location) then
        return nil, errType.codes.InvalidParams,
        {message = 'Invalid arcview configuration.'}
    end

    if args ~= nil then
        if utils.isArray(args) then
            database = args[1]
            host = args[2]
            port = args[3]
            user = args[4]
            password = args[5]
            name = args[6]
            schema = args[7]
            route_api = args[8]
        else
            database = args.database
            host = args.host
            port = args.port
            user = args.user
            password = args.password
            name = args.name
            schema = args.schema
            route_api = args.route_api
        end
    end

    if database == nil or type(database) ~= 'string' then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'database\' is mandatory and must be a string.'}
    end
    if host ~= nil and type(host) ~= 'string' then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'host\' must be a string.'}
    end
    if user ~= nil and type(user) ~= 'string' then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'user\' must be a string.'}
    end
    if port ~= nil and utils.isInt(port) == false then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'port\' must be an int.'}
    end
    if password ~= nil and type(password) ~= 'string' then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'password\' must be a string.'}
    end
    if name ~= nil and type(name) ~= 'string' then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'name\' must be a string.'}
    end
    if schema ~= nil and type(schema) ~= 'string' then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'schema\' must be a string.'}
    end
    if route_api ~= nil and type(route_api) ~= 'string' then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'route_api\' must be a string.'}
    end

    local rexarcParams = {}

    if database ~= nil then
        table.insert(rexarcParams, '--database')
        table.insert(rexarcParams, database)
    end
    if host ~= nil then
        table.insert(rexarcParams, '--host')
        table.insert(rexarcParams, host)
    end
    if port ~= nil then
        table.insert(rexarcParams, '--port')
        table.insert(rexarcParams, tostring(port))
    end
    if user ~= nil then
        table.insert(rexarcParams, '--user')
        table.insert(rexarcParams, user)
    end
    if password ~= nil then
        table.insert(rexarcParams, '--password')
        table.insert(rexarcParams, password)
    end
    if name ~= nil then
        table.insert(rexarcParams, '--name')
        table.insert(rexarcParams, name)
    end
    if schema ~= nil then
        table.insert(rexarcParams, '--schema')
        table.insert(rexarcParams, schema)
    end
    if route_api ~= nil then
        table.insert(rexarcParams, '--route_api')
        table.insert(rexarcParams, route_api)
    end

    local app_dir = config.arcview.location
    local out, err, errData = utils.shell_posix('sudo', nil, '/usr/bin/python3', app_dir .. 'manage_bootstrap.py', 'database_generic_configuration_create', unpack(rexarcParams))
    if err ~= nil then
        return nil, err, errData
    end

    return out
end

return ArcView_DbConfigCreate
