local errType = require 'errType'
local utils = require 'utils'

local Cpu_GetState = {}

function Cpu_GetState.parseLine(info, line)
    local param, value = string.match(line, '(.+):(.+)')
    if param ~= nil and value ~= nil then
        param = utils.trim(param)
        value = utils.trim(value)
        if param == 'Hardware' then
            info.hardware = value
            return
        end
        if param == 'Serial' then
            info.serial = value
            return
        end
        if param == 'processor' then
            info.cpu = tonumber(value) + 1
            return
        end
        if param == 'cpu MHz' then
            info.cpuMHz = tonumber(value)
            return
        end
        if param == 'model name' then
            info.model = value
            return
        end
    end
end

function Cpu_GetState.invoke()

    local content = utils.readFromFile('/proc/cpuinfo')
    if content == nil then
        return nil, errType.codes.ServerError, {message = 'Failed to read content from path \'/proc/cpuinfo\'.'}
    end

    local cpu = {}
    local lines = utils.split(content, '\n')
    for _, line in pairs(lines) do
        Cpu_GetState.parseLine(cpu, line)
    end

    return cpu
end

return Cpu_GetState
