local utils = require 'utils'

local Disks_GetState = {}

function Disks_GetState.invoke()
    local stdout, err, errData = utils.shell_posix('df', nil, '-T')
    if err ~= nil then
        return nil, err, errData
    end

    local lines = utils.split(stdout, '\n')

    local fileSystems = {}

    for _, line in pairs(lines) do
        local filesystem, type, blocks, usedBlocks, available, used, mounted = string.match(line,
            "(%S+)%s+(%S+)%s+(%d+)%s+(%d+)%s+(%d+)%s+(%d+)%%%s+(.+)")

        if filesystem ~= nil then
            local fs = {}
            fs.filesystem = filesystem
            fs.blocks = blocks
            fs.usedBlocks = usedBlocks
            fs.available = available
            fs.usedPercent = used
            fs.mounted = utils.trim(mounted)
            fs.type = type

            table.insert(fileSystems, fs)
        end
    end

    for _, fs in pairs(fileSystems) do
        stdout, err = utils.shell_posix('blkid', nil, fs.filesystem)
        if err == nil then
            local label = string.match(stdout, "LABEL=\"([^\"]+)\"")
            fs.label = label
        end
    end

    return fileSystems
end

return Disks_GetState
