local errType = require 'errType'
local utils = require 'utils'

local FileSystem_Copy = {}

function FileSystem_Copy.invoke(args)
    local source
    local destination

    if args ~= nil then
        if utils.isArray(args) then
            if #args >= 1 then
                source = args[1]
            end
            if #args >= 2 then
                destination = args[2]
            end
        else
            source = args.source
            destination = args.destination
        end
    end

    if source == nil or type(source) ~= 'string' then
        return nil, errType.codes.InvalidParams, {message = 'Failed to read parameter \'source\'.'}
    end
    if destination == nil or type(destination) ~= 'string' then
        return nil, errType.codes.InvalidParams, {message = 'Failed to read parameter \'destination\'.'}
    end

    local readable, err, errData = utils.isPathReadable(source)
    if readable ~= true then
        return nil, err, errData
    end

    local writable
    writable, err, errData = utils.isPathWritable(destination)
    if writable ~= true then
        return nil, err, errData
    end

    local _
    _, err, errData = utils.shell_posix('cp', nil, '-R', source, destination)
    if err ~= nil then
        return nil, err, errData
    end
end

return FileSystem_Copy
