local errType = require 'errType'
local utils = require 'utils'
local Network_GetState = require 'methods/Network_GetState'

local Network_RemoveConfig = {}

function Network_RemoveConfig.invoke(args)
    local ifaceName

    if args ~= nil then
        if utils.isArray(args) then
            if #args >= 1 then
                ifaceName = args[1]
            end
        else
            ifaceName = args.interface
        end
    end

    if ifaceName == nil or type(ifaceName) ~= 'string' then
        return nil, errType.codes.InvalidParams, {message = 'Failed to read parameter \'interface\'.'}
    end

    local interfaces = Network_GetState.invoke()
    if interfaces == nil then
        return nil, errType.codes.InternalError, {message = 'Failed to query interfaces.'}
    end

    local interface
    for _, iface in ipairs(interfaces) do
        if iface.name == ifaceName then
            interface = iface
        end
    end
    if interface == nil then
        return nil, errType.codes.InvalidParams, {message = 'Interface \'' .. ifaceName .. '\' not found.'}
    end

    if interface.managed then
        local fileName = '/etc/systemd/network/' .. interface.name .. '.network'
        if utils.fileExists(fileName) then
            local _, err = utils.shell_posix('sudo', nil, 'rm', fileName)
            if err ~= nil then
                return nil, errType.codes.ServerError, {message = 'Failed to remove file ' .. fileName .. '.'}
            end
        end
        if interface.managedByHwAddr then
            fileName = '/etc/systemd/network/' .. Network_GetState.hwaddr2name(interface.hwaddr) .. '.network'
            if utils.fileExists(fileName) then
                local _, err = utils.shell_posix('sudo', nil, 'rm', fileName)
                if err ~= nil then
                    return nil, errType.codes.ServerError, {message = 'Failed to remove file ' .. fileName .. '.'}
                end
            end
        end
    end
end

return Network_RemoveConfig
