local errType = require 'errType'
local utils = require 'utils'
local Network_GetState = require 'methods/Network_GetState'

local Network_ScanWifi = {}

function Network_ScanWifi.invoke(args)
    local interfaceName

    if args ~= nil then
        if utils.isArray(args) then
            interfaceName = args[1]
        else
            interfaceName = args.interface
        end
    end

    if interfaceName == nil or type(interfaceName) ~= 'string' then
        return nil, errType.codes.InvalidParams, {message = 'Failed to read parameter \'interface\'.'}
    end

    local interfaces = Network_GetState.invoke()
    if interfaces == nil then
        return nil, errType.codes.InternalError, {message = 'Failed to query interfaces.'}
    end

    local interface
    for _, iface in ipairs(interfaces) do
        if iface.name == interfaceName then
            interface = iface
        end
    end
    if interface == nil then
        return nil, errType.codes.InvalidParams,
        {message = 'Interface \'' .. interfaceName .. '\' not found.'}
    end
    if interface.wireless ~= true then
        return nil, errType.codes.InvalidParams,
        {message = 'Interface \'' .. interfaceName .. '\' is not a wireless interface.'}
    end

    local stdout, err, errData = utils.shell_posix('iwlist', nil, interfaceName, 'scanning')
    if err ~= nil then
        return nil, err, errData
    end

    local lines = utils.split(stdout, '\n')
    local networks = {}
    local network = nil
    for _, line in pairs(lines) do
        local address = string.match(line, '%s*Cell%s+%d+%s*-%s*Address:%s*([%w%d:]+)%s*')
        if address ~= nil then
            network = {}
            network.hwaddr = address
            table.insert(networks, network)
        end

        if network ~= nil then
            local ssid= string.match(line, '%s*ESSID:%s*\"(.+)\"')
            if ssid ~= nil then
                network.ssid = ssid
            end
            local channel = string.match(line, '%s*Channel:%s*(%d+)')
            if channel ~= nil then
                network.channel = channel
            end
            local qv, qmax = string.match(line, '%s*Quality%s*=%s*(%d+)/(%d+)%s*')
            if qv ~= nil and qmax ~= nil then
                network.quality = math.floor((tonumber(qv)/tonumber(qmax)) * 100)
            end
            local signalLevel = string.match(line, '%s*Signal level%s*=%s*([%+-%d]+)')
            if signalLevel ~= nil then
                network.signal_level = tonumber(signalLevel)
            end
            local freq= string.match(line, '%s*Frequency%s*:%s*([,%.%d]+)')
            if freq ~= nil then
                network.frequency = tonumber(freq)
            end
        end
    end

    return networks
end

return Network_ScanWifi
