local errType = require 'errType'
local utils = require 'utils'

local OSInfo_GetState = {}

function OSInfo_GetState.parseLine(info, line)
    local param, value = string.match(line, '(.+)=(.+)')
    if param ~= nil and value ~= nil then
        param = utils.trim(param)
        value = utils.trim(value)
        if param == 'NAME' then
            info.name = utils.unquote(value)
            return
        end
        if param == 'VERSION' then
            info.version = utils.unquote(value)
            return
        end
        if param == 'PTXDIST_BSP_NAME' then
            info.name = utils.unquote(value)
            return
        end
        if param == 'PTXDIST_BSP_VENDOR' then
            info.version = utils.unquote(value)
            return
        end
        if param == 'PTXDIST_BSP_VERSION' then
            info.version = info.version .. utils.unquote(value)
            return
        end
        if param == 'PTXDIST_BUILD_DATE' then
            info.version = info.version .. " " .. utils.unquote(value)
            return
        end
    end
end

function OSInfo_GetState.invoke()

    local content = utils.readFromFile('/etc/os-release')
    if content == nil then
        return nil, errType.codes.ServerError, {message = 'Failed to read content from path \'/etc/os-release\'.'}
    end


    local osinfo = {}
    local lines = utils.split(content, '\n')
    for _, line in pairs(lines) do
        OSInfo_GetState.parseLine(osinfo, line)
    end

    return osinfo
end

return OSInfo_GetState
