local cjson = require 'cjson.safe'
local errType = require 'errType'
local utils = require 'utils'

local RexArc_Read = {}

function RexArc_Read.invoke(args)
    local arcName, startOfTimespan, endOfTimespan, itmCntLimit,
    itmTotalCnt, idFilter, fNoData, fNoEvents, fNoLog,
    inputDirPth, csv, output, delimiter, columnFilter, customHeader

    if args ~= nil then
        if utils.isArray(args) then
            arcName = args[1]
            startOfTimespan = args[2]
            endOfTimespan = args[3]
            itmCntLimit = args[4]
            itmTotalCnt = args[5]
            idFilter = args[6]
            fNoData = args[7]
            fNoEvents = args[8]
            fNoLog = args[9]
            csv = args[10]
            output = args[11]
            inputDirPth = args[12]
            delimiter = args[13]
            columnFilter = args[14]
            customHeader = args[15]
        else
            arcName = args.arcName
            startOfTimespan = args.startOfTimespan
            endOfTimespan = args.endOfTimespan
            itmCntLimit = args.itmCntLimit
            itmTotalCnt = args.itmTotalCnt
            idFilter = args.idFilter
            fNoData = args.fNoData
            fNoEvents = args.fNoEvents
            fNoLog = args.fNoLog
            inputDirPth = args.inputDirPth
            csv = args.csv
            output = args.output
            delimiter = args.delimiter
            columnFilter = args.columnFilter
            customHeader = args.customHeader

        end
    end

    if arcName == nil or type(arcName) ~= 'string' then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'arcName\' is mandatory and must be a string.'}
    end
    if startOfTimespan ~= nil and type(startOfTimespan) ~= 'string' then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'startOfTimespan\' must be a string.'}
    end
    if endOfTimespan ~= nil and type(endOfTimespan) ~= 'string' then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'endOfTimespan\' must be a string.'}
    end
    if itmCntLimit ~= nil and utils.isInt(itmCntLimit) == false then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'itmCntLimit\' must be an int.'}
    end
    if itmTotalCnt ~= nil and utils.isBool(itmTotalCnt) == false then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'itmTotalCnt\' must be a boolean.'}
    end
    if idFilter ~= nil and type(idFilter) ~= 'string' then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'idFilter\' must be a string in format e.g. \'1..5,8,10..20\'.'}
    end
    if fNoData ~= nil and utils.isBool(fNoData) == false then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'fNoData\' must be a boolean.'}
    end
    if fNoEvents ~= nil and utils.isBool(fNoEvents) == false then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'fNoEvents\' must be a boolean.'}
    end
    if fNoLog ~= nil and utils.isBool(fNoLog) == false then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'fNoLog\' must be a boolean.'}
    end
    if inputDirPth ~= nil and type(inputDirPth) ~= 'string' then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'inputDirPth\' must be a string.'}
    end
    if output ~= nil and type(output) ~= 'string' then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'output\' must be a string.'}
    end
    if csv ~= nil and utils.isBool(csv) == false then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'csv\' must be a boolean.'}
    end
    if csv ~= nil and delimiter ~= nil and  type(delimiter) ~= 'string' then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'delimiter\' must be a string.'}
    end
    if csv ~= nil and delimiter ~= nil then
        if delimiter ~= 'comma' and delimiter ~= 'semicolon' and delimiter ~= 'space' and delimiter ~= 'tab' then
            return nil, errType.codes.InvalidParams,
            {message = 'Parameter \'delimiter\' must be \'comma\', \'semicolon\' or \'space\'.'}
        end
    end
    if csv ~= nil and columnFilter ~= nil and  type(columnFilter) ~= 'string' then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'columnFilter\' must be a string.'}
    end
    if csv ~= nil and customHeader ~= nil and  type(customHeader) ~= 'string' then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'customHeader\' must be a string.'}
    end


    local rexarcParams = {}

    if startOfTimespan ~= nil then
        table.insert(rexarcParams, '--from-timestamp')
        table.insert(rexarcParams, startOfTimespan)
    end
    if endOfTimespan ~= nil then
        table.insert(rexarcParams, '--until-timestamp')
        table.insert(rexarcParams, endOfTimespan)
    end
    if itmCntLimit ~= nil then
        table.insert(rexarcParams, '--item-count')
        table.insert(rexarcParams, tostring(itmCntLimit))
    end
    if itmTotalCnt == true then
        table.insert(rexarcParams, '--item-total-count')
    end
    if idFilter ~= nil then
        table.insert(rexarcParams, '--filter-id')
        table.insert(rexarcParams, idFilter)
    end
    if fNoData == true then
        table.insert(rexarcParams, '--no-data')
    end
    if fNoEvents == true then
        table.insert(rexarcParams, '--no-events')
    end
    if fNoLog == true then
        table.insert(rexarcParams, '--no-log')
    end
    if inputDirPth ~= nil then
        table.insert(rexarcParams, '--input-dir')
        table.insert(rexarcParams, inputDirPth)
    end
    if output ~= nil then
        table.insert(rexarcParams, '--output')
        table.insert(rexarcParams, output)
    end
    if csv == true then
        table.insert(rexarcParams, '--csv')
    end
    if csv == true and delimiter ~= nil then
        if delimiter == 'semicolon' then
            table.insert(rexarcParams, '--delimiter-semicol')
        elseif delimiter == 'space' then
            table.insert(rexarcParams, '--delimiter-space')
        elseif delimiter == 'tab' then
            table.insert(rexarcParams, '--delimiter-tab')
        end
    end
    if csv == true and columnFilter ~= nil then
        table.insert(rexarcParams, '--csv-column-filter')
        table.insert(rexarcParams, columnFilter)
    end
    if csv == true and customHeader ~= nil then
        table.insert(rexarcParams, '--csv-header')
        table.insert(rexarcParams, customHeader)
    end

    table.insert(rexarcParams, arcName)

    print(unpack(rexarcParams))

    local out, err, errData = utils.shell_posix('RexArc', nil, unpack(rexarcParams))

    if errData ~= nil and errData.exitcode == 254 then
        -- no data found - continue as if no error occurred
        err = nil
    end

    if err ~= nil then
        return nil, err, errData
    end

    local data
    if csv == true or output == true then
        data = out;
    else
        data = cjson.decode(out)
    end

    print(data)
    if csv ~= true and output ~= true and (data == nil or type(data) ~= 'table') then
        return nil, errType.codes.InternalError,
        {message = 'Failed to parse output of RexArc to JSON.'}
    end
    return data
end

return RexArc_Read
